@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.7.6.  */
+/* A Bison parser, made by GNU Bison 3.8.2.  */
 
 /* Bison interface for Yacc-like parsers in C
 
@@ -115,85 +115,89 @@ extern int c_debug;
     VAR_TLS_SERVICE_OCSP = 316,    /* VAR_TLS_SERVICE_OCSP  */
     VAR_TLS_PORT = 317,            /* VAR_TLS_PORT  */
     VAR_TLS_CERT_BUNDLE = 318,     /* VAR_TLS_CERT_BUNDLE  */
-    VAR_CPU_AFFINITY = 319,        /* VAR_CPU_AFFINITY  */
-    VAR_XFRD_CPU_AFFINITY = 320,   /* VAR_XFRD_CPU_AFFINITY  */
-    VAR_SERVER_CPU_AFFINITY = 321, /* VAR_SERVER_CPU_AFFINITY  */
-    VAR_DROP_UPDATES = 322,        /* VAR_DROP_UPDATES  */
-    VAR_XFRD_TCP_MAX = 323,        /* VAR_XFRD_TCP_MAX  */
-    VAR_XFRD_TCP_PIPELINE = 324,   /* VAR_XFRD_TCP_PIPELINE  */
-    VAR_DNSTAP = 325,              /* VAR_DNSTAP  */
-    VAR_DNSTAP_ENABLE = 326,       /* VAR_DNSTAP_ENABLE  */
-    VAR_DNSTAP_SOCKET_PATH = 327,  /* VAR_DNSTAP_SOCKET_PATH  */
-    VAR_DNSTAP_IP = 328,           /* VAR_DNSTAP_IP  */
-    VAR_DNSTAP_TLS = 329,          /* VAR_DNSTAP_TLS  */
-    VAR_DNSTAP_TLS_SERVER_NAME = 330, /* VAR_DNSTAP_TLS_SERVER_NAME  */
-    VAR_DNSTAP_TLS_CERT_BUNDLE = 331, /* VAR_DNSTAP_TLS_CERT_BUNDLE  */
-    VAR_DNSTAP_TLS_CLIENT_KEY_FILE = 332, /* VAR_DNSTAP_TLS_CLIENT_KEY_FILE  */
-    VAR_DNSTAP_TLS_CLIENT_CERT_FILE = 333, /* VAR_DNSTAP_TLS_CLIENT_CERT_FILE  */
-    VAR_DNSTAP_SEND_IDENTITY = 334, /* VAR_DNSTAP_SEND_IDENTITY  */
-    VAR_DNSTAP_SEND_VERSION = 335, /* VAR_DNSTAP_SEND_VERSION  */
-    VAR_DNSTAP_IDENTITY = 336,     /* VAR_DNSTAP_IDENTITY  */
-    VAR_DNSTAP_VERSION = 337,      /* VAR_DNSTAP_VERSION  */
-    VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES = 338, /* VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES  */
-    VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES = 339, /* VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES  */
-    VAR_REMOTE_CONTROL = 340,      /* VAR_REMOTE_CONTROL  */
-    VAR_CONTROL_ENABLE = 341,      /* VAR_CONTROL_ENABLE  */
-    VAR_CONTROL_INTERFACE = 342,   /* VAR_CONTROL_INTERFACE  */
-    VAR_CONTROL_PORT = 343,        /* VAR_CONTROL_PORT  */
-    VAR_SERVER_KEY_FILE = 344,     /* VAR_SERVER_KEY_FILE  */
-    VAR_SERVER_CERT_FILE = 345,    /* VAR_SERVER_CERT_FILE  */
-    VAR_CONTROL_KEY_FILE = 346,    /* VAR_CONTROL_KEY_FILE  */
-    VAR_CONTROL_CERT_FILE = 347,   /* VAR_CONTROL_CERT_FILE  */
-    VAR_KEY = 348,                 /* VAR_KEY  */
-    VAR_ALGORITHM = 349,           /* VAR_ALGORITHM  */
-    VAR_SECRET = 350,              /* VAR_SECRET  */
-    VAR_TLS_AUTH = 351,            /* VAR_TLS_AUTH  */
-    VAR_TLS_AUTH_DOMAIN_NAME = 352, /* VAR_TLS_AUTH_DOMAIN_NAME  */
-    VAR_TLS_AUTH_CLIENT_CERT = 353, /* VAR_TLS_AUTH_CLIENT_CERT  */
-    VAR_TLS_AUTH_CLIENT_KEY = 354, /* VAR_TLS_AUTH_CLIENT_KEY  */
-    VAR_TLS_AUTH_CLIENT_KEY_PW = 355, /* VAR_TLS_AUTH_CLIENT_KEY_PW  */
-    VAR_PATTERN = 356,             /* VAR_PATTERN  */
-    VAR_NAME = 357,                /* VAR_NAME  */
-    VAR_ZONEFILE = 358,            /* VAR_ZONEFILE  */
-    VAR_NOTIFY = 359,              /* VAR_NOTIFY  */
-    VAR_PROVIDE_XFR = 360,         /* VAR_PROVIDE_XFR  */
-    VAR_ALLOW_QUERY = 361,         /* VAR_ALLOW_QUERY  */
-    VAR_AXFR = 362,                /* VAR_AXFR  */
-    VAR_UDP = 363,                 /* VAR_UDP  */
-    VAR_NOTIFY_RETRY = 364,        /* VAR_NOTIFY_RETRY  */
-    VAR_ALLOW_NOTIFY = 365,        /* VAR_ALLOW_NOTIFY  */
-    VAR_REQUEST_XFR = 366,         /* VAR_REQUEST_XFR  */
-    VAR_ALLOW_AXFR_FALLBACK = 367, /* VAR_ALLOW_AXFR_FALLBACK  */
-    VAR_OUTGOING_INTERFACE = 368,  /* VAR_OUTGOING_INTERFACE  */
-    VAR_ANSWER_COOKIE = 369,       /* VAR_ANSWER_COOKIE  */
-    VAR_COOKIE_SECRET = 370,       /* VAR_COOKIE_SECRET  */
-    VAR_COOKIE_SECRET_FILE = 371,  /* VAR_COOKIE_SECRET_FILE  */
-    VAR_MAX_REFRESH_TIME = 372,    /* VAR_MAX_REFRESH_TIME  */
-    VAR_MIN_REFRESH_TIME = 373,    /* VAR_MIN_REFRESH_TIME  */
-    VAR_MAX_RETRY_TIME = 374,      /* VAR_MAX_RETRY_TIME  */
-    VAR_MIN_RETRY_TIME = 375,      /* VAR_MIN_RETRY_TIME  */
-    VAR_MIN_EXPIRE_TIME = 376,     /* VAR_MIN_EXPIRE_TIME  */
-    VAR_MULTI_MASTER_CHECK = 377,  /* VAR_MULTI_MASTER_CHECK  */
-    VAR_SIZE_LIMIT_XFR = 378,      /* VAR_SIZE_LIMIT_XFR  */
-    VAR_ZONESTATS = 379,           /* VAR_ZONESTATS  */
-    VAR_INCLUDE_PATTERN = 380,     /* VAR_INCLUDE_PATTERN  */
-    VAR_STORE_IXFR = 381,          /* VAR_STORE_IXFR  */
-    VAR_IXFR_SIZE = 382,           /* VAR_IXFR_SIZE  */
-    VAR_IXFR_NUMBER = 383,         /* VAR_IXFR_NUMBER  */
-    VAR_CREATE_IXFR = 384,         /* VAR_CREATE_IXFR  */
-    VAR_ZONE = 385,                /* VAR_ZONE  */
-    VAR_RRL_WHITELIST = 386,       /* VAR_RRL_WHITELIST  */
-    VAR_SERVERS = 387,             /* VAR_SERVERS  */
-    VAR_BINDTODEVICE = 388,        /* VAR_BINDTODEVICE  */
-    VAR_SETFIB = 389,              /* VAR_SETFIB  */
-    VAR_VERIFY = 390,              /* VAR_VERIFY  */
-    VAR_ENABLE = 391,              /* VAR_ENABLE  */
-    VAR_VERIFY_ZONE = 392,         /* VAR_VERIFY_ZONE  */
-    VAR_VERIFY_ZONES = 393,        /* VAR_VERIFY_ZONES  */
-    VAR_VERIFIER = 394,            /* VAR_VERIFIER  */
-    VAR_VERIFIER_COUNT = 395,      /* VAR_VERIFIER_COUNT  */
-    VAR_VERIFIER_FEED_ZONE = 396,  /* VAR_VERIFIER_FEED_ZONE  */
-    VAR_VERIFIER_TIMEOUT = 397     /* VAR_VERIFIER_TIMEOUT  */
+    VAR_PROXY_PROTOCOL_PORT = 319, /* VAR_PROXY_PROTOCOL_PORT  */
+    VAR_CPU_AFFINITY = 320,        /* VAR_CPU_AFFINITY  */
+    VAR_XFRD_CPU_AFFINITY = 321,   /* VAR_XFRD_CPU_AFFINITY  */
+    VAR_SERVER_CPU_AFFINITY = 322, /* VAR_SERVER_CPU_AFFINITY  */
+    VAR_DROP_UPDATES = 323,        /* VAR_DROP_UPDATES  */
+    VAR_XFRD_TCP_MAX = 324,        /* VAR_XFRD_TCP_MAX  */
+    VAR_XFRD_TCP_PIPELINE = 325,   /* VAR_XFRD_TCP_PIPELINE  */
+    VAR_DNSTAP = 326,              /* VAR_DNSTAP  */
+    VAR_DNSTAP_ENABLE = 327,       /* VAR_DNSTAP_ENABLE  */
+    VAR_DNSTAP_SOCKET_PATH = 328,  /* VAR_DNSTAP_SOCKET_PATH  */
+    VAR_DNSTAP_IP = 329,           /* VAR_DNSTAP_IP  */
+    VAR_DNSTAP_TLS = 330,          /* VAR_DNSTAP_TLS  */
+    VAR_DNSTAP_TLS_SERVER_NAME = 331, /* VAR_DNSTAP_TLS_SERVER_NAME  */
+    VAR_DNSTAP_TLS_CERT_BUNDLE = 332, /* VAR_DNSTAP_TLS_CERT_BUNDLE  */
+    VAR_DNSTAP_TLS_CLIENT_KEY_FILE = 333, /* VAR_DNSTAP_TLS_CLIENT_KEY_FILE  */
+    VAR_DNSTAP_TLS_CLIENT_CERT_FILE = 334, /* VAR_DNSTAP_TLS_CLIENT_CERT_FILE  */
+    VAR_DNSTAP_SEND_IDENTITY = 335, /* VAR_DNSTAP_SEND_IDENTITY  */
+    VAR_DNSTAP_SEND_VERSION = 336, /* VAR_DNSTAP_SEND_VERSION  */
+    VAR_DNSTAP_IDENTITY = 337,     /* VAR_DNSTAP_IDENTITY  */
+    VAR_DNSTAP_VERSION = 338,      /* VAR_DNSTAP_VERSION  */
+    VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES = 339, /* VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES  */
+    VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES = 340, /* VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES  */
+    VAR_REMOTE_CONTROL = 341,      /* VAR_REMOTE_CONTROL  */
+    VAR_CONTROL_ENABLE = 342,      /* VAR_CONTROL_ENABLE  */
+    VAR_CONTROL_INTERFACE = 343,   /* VAR_CONTROL_INTERFACE  */
+    VAR_CONTROL_PORT = 344,        /* VAR_CONTROL_PORT  */
+    VAR_SERVER_KEY_FILE = 345,     /* VAR_SERVER_KEY_FILE  */
+    VAR_SERVER_CERT_FILE = 346,    /* VAR_SERVER_CERT_FILE  */
+    VAR_CONTROL_KEY_FILE = 347,    /* VAR_CONTROL_KEY_FILE  */
+    VAR_CONTROL_CERT_FILE = 348,   /* VAR_CONTROL_CERT_FILE  */
+    VAR_KEY = 349,                 /* VAR_KEY  */
+    VAR_ALGORITHM = 350,           /* VAR_ALGORITHM  */
+    VAR_SECRET = 351,              /* VAR_SECRET  */
+    VAR_TLS_AUTH = 352,            /* VAR_TLS_AUTH  */
+    VAR_TLS_AUTH_DOMAIN_NAME = 353, /* VAR_TLS_AUTH_DOMAIN_NAME  */
+    VAR_TLS_AUTH_CLIENT_CERT = 354, /* VAR_TLS_AUTH_CLIENT_CERT  */
+    VAR_TLS_AUTH_CLIENT_KEY = 355, /* VAR_TLS_AUTH_CLIENT_KEY  */
+    VAR_TLS_AUTH_CLIENT_KEY_PW = 356, /* VAR_TLS_AUTH_CLIENT_KEY_PW  */
+    VAR_PATTERN = 357,             /* VAR_PATTERN  */
+    VAR_NAME = 358,                /* VAR_NAME  */
+    VAR_ZONEFILE = 359,            /* VAR_ZONEFILE  */
+    VAR_NOTIFY = 360,              /* VAR_NOTIFY  */
+    VAR_PROVIDE_XFR = 361,         /* VAR_PROVIDE_XFR  */
+    VAR_ALLOW_QUERY = 362,         /* VAR_ALLOW_QUERY  */
+    VAR_AXFR = 363,                /* VAR_AXFR  */
+    VAR_UDP = 364,                 /* VAR_UDP  */
+    VAR_NOTIFY_RETRY = 365,        /* VAR_NOTIFY_RETRY  */
+    VAR_ALLOW_NOTIFY = 366,        /* VAR_ALLOW_NOTIFY  */
+    VAR_REQUEST_XFR = 367,         /* VAR_REQUEST_XFR  */
+    VAR_ALLOW_AXFR_FALLBACK = 368, /* VAR_ALLOW_AXFR_FALLBACK  */
+    VAR_OUTGOING_INTERFACE = 369,  /* VAR_OUTGOING_INTERFACE  */
+    VAR_ANSWER_COOKIE = 370,       /* VAR_ANSWER_COOKIE  */
+    VAR_COOKIE_SECRET = 371,       /* VAR_COOKIE_SECRET  */
+    VAR_COOKIE_SECRET_FILE = 372,  /* VAR_COOKIE_SECRET_FILE  */
+    VAR_MAX_REFRESH_TIME = 373,    /* VAR_MAX_REFRESH_TIME  */
+    VAR_MIN_REFRESH_TIME = 374,    /* VAR_MIN_REFRESH_TIME  */
+    VAR_MAX_RETRY_TIME = 375,      /* VAR_MAX_RETRY_TIME  */
+    VAR_MIN_RETRY_TIME = 376,      /* VAR_MIN_RETRY_TIME  */
+    VAR_MIN_EXPIRE_TIME = 377,     /* VAR_MIN_EXPIRE_TIME  */
+    VAR_MULTI_PRIMARY_CHECK = 378, /* VAR_MULTI_PRIMARY_CHECK  */
+    VAR_SIZE_LIMIT_XFR = 379,      /* VAR_SIZE_LIMIT_XFR  */
+    VAR_ZONESTATS = 380,           /* VAR_ZONESTATS  */
+    VAR_INCLUDE_PATTERN = 381,     /* VAR_INCLUDE_PATTERN  */
+    VAR_STORE_IXFR = 382,          /* VAR_STORE_IXFR  */
+    VAR_IXFR_SIZE = 383,           /* VAR_IXFR_SIZE  */
+    VAR_IXFR_NUMBER = 384,         /* VAR_IXFR_NUMBER  */
+    VAR_CREATE_IXFR = 385,         /* VAR_CREATE_IXFR  */
+    VAR_CATALOG = 386,             /* VAR_CATALOG  */
+    VAR_CATALOG_MEMBER_PATTERN = 387, /* VAR_CATALOG_MEMBER_PATTERN  */
+    VAR_CATALOG_PRODUCER_ZONE = 388, /* VAR_CATALOG_PRODUCER_ZONE  */
+    VAR_ZONE = 389,                /* VAR_ZONE  */
+    VAR_RRL_WHITELIST = 390,       /* VAR_RRL_WHITELIST  */
+    VAR_SERVERS = 391,             /* VAR_SERVERS  */
+    VAR_BINDTODEVICE = 392,        /* VAR_BINDTODEVICE  */
+    VAR_SETFIB = 393,              /* VAR_SETFIB  */
+    VAR_VERIFY = 394,              /* VAR_VERIFY  */
+    VAR_ENABLE = 395,              /* VAR_ENABLE  */
+    VAR_VERIFY_ZONE = 396,         /* VAR_VERIFY_ZONE  */
+    VAR_VERIFY_ZONES = 397,        /* VAR_VERIFY_ZONES  */
+    VAR_VERIFIER = 398,            /* VAR_VERIFIER  */
+    VAR_VERIFIER_COUNT = 399,      /* VAR_VERIFIER_COUNT  */
+    VAR_VERIFIER_FEED_ZONE = 400,  /* VAR_VERIFIER_FEED_ZONE  */
+    VAR_VERIFIER_TIMEOUT = 401     /* VAR_VERIFIER_TIMEOUT  */
   };
   typedef enum yytokentype yytoken_kind_t;
 #endif
@@ -263,91 +267,95 @@ extern int c_debug;
 #define VAR_TLS_SERVICE_OCSP 316
 #define VAR_TLS_PORT 317
 #define VAR_TLS_CERT_BUNDLE 318
-#define VAR_CPU_AFFINITY 319
-#define VAR_XFRD_CPU_AFFINITY 320
-#define VAR_SERVER_CPU_AFFINITY 321
-#define VAR_DROP_UPDATES 322
-#define VAR_XFRD_TCP_MAX 323
-#define VAR_XFRD_TCP_PIPELINE 324
-#define VAR_DNSTAP 325
-#define VAR_DNSTAP_ENABLE 326
-#define VAR_DNSTAP_SOCKET_PATH 327
-#define VAR_DNSTAP_IP 328
-#define VAR_DNSTAP_TLS 329
-#define VAR_DNSTAP_TLS_SERVER_NAME 330
-#define VAR_DNSTAP_TLS_CERT_BUNDLE 331
-#define VAR_DNSTAP_TLS_CLIENT_KEY_FILE 332
-#define VAR_DNSTAP_TLS_CLIENT_CERT_FILE 333
-#define VAR_DNSTAP_SEND_IDENTITY 334
-#define VAR_DNSTAP_SEND_VERSION 335
-#define VAR_DNSTAP_IDENTITY 336
-#define VAR_DNSTAP_VERSION 337
-#define VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES 338
-#define VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES 339
-#define VAR_REMOTE_CONTROL 340
-#define VAR_CONTROL_ENABLE 341
-#define VAR_CONTROL_INTERFACE 342
-#define VAR_CONTROL_PORT 343
-#define VAR_SERVER_KEY_FILE 344
-#define VAR_SERVER_CERT_FILE 345
-#define VAR_CONTROL_KEY_FILE 346
-#define VAR_CONTROL_CERT_FILE 347
-#define VAR_KEY 348
-#define VAR_ALGORITHM 349
-#define VAR_SECRET 350
-#define VAR_TLS_AUTH 351
-#define VAR_TLS_AUTH_DOMAIN_NAME 352
-#define VAR_TLS_AUTH_CLIENT_CERT 353
-#define VAR_TLS_AUTH_CLIENT_KEY 354
-#define VAR_TLS_AUTH_CLIENT_KEY_PW 355
-#define VAR_PATTERN 356
-#define VAR_NAME 357
-#define VAR_ZONEFILE 358
-#define VAR_NOTIFY 359
-#define VAR_PROVIDE_XFR 360
-#define VAR_ALLOW_QUERY 361
-#define VAR_AXFR 362
-#define VAR_UDP 363
-#define VAR_NOTIFY_RETRY 364
-#define VAR_ALLOW_NOTIFY 365
-#define VAR_REQUEST_XFR 366
-#define VAR_ALLOW_AXFR_FALLBACK 367
-#define VAR_OUTGOING_INTERFACE 368
-#define VAR_ANSWER_COOKIE 369
-#define VAR_COOKIE_SECRET 370
-#define VAR_COOKIE_SECRET_FILE 371
-#define VAR_MAX_REFRESH_TIME 372
-#define VAR_MIN_REFRESH_TIME 373
-#define VAR_MAX_RETRY_TIME 374
-#define VAR_MIN_RETRY_TIME 375
-#define VAR_MIN_EXPIRE_TIME 376
-#define VAR_MULTI_MASTER_CHECK 377
-#define VAR_SIZE_LIMIT_XFR 378
-#define VAR_ZONESTATS 379
-#define VAR_INCLUDE_PATTERN 380
-#define VAR_STORE_IXFR 381
-#define VAR_IXFR_SIZE 382
-#define VAR_IXFR_NUMBER 383
-#define VAR_CREATE_IXFR 384
-#define VAR_ZONE 385
-#define VAR_RRL_WHITELIST 386
-#define VAR_SERVERS 387
-#define VAR_BINDTODEVICE 388
-#define VAR_SETFIB 389
-#define VAR_VERIFY 390
-#define VAR_ENABLE 391
-#define VAR_VERIFY_ZONE 392
-#define VAR_VERIFY_ZONES 393
-#define VAR_VERIFIER 394
-#define VAR_VERIFIER_COUNT 395
-#define VAR_VERIFIER_FEED_ZONE 396
-#define VAR_VERIFIER_TIMEOUT 397
+#define VAR_PROXY_PROTOCOL_PORT 319
+#define VAR_CPU_AFFINITY 320
+#define VAR_XFRD_CPU_AFFINITY 321
+#define VAR_SERVER_CPU_AFFINITY 322
+#define VAR_DROP_UPDATES 323
+#define VAR_XFRD_TCP_MAX 324
+#define VAR_XFRD_TCP_PIPELINE 325
+#define VAR_DNSTAP 326
+#define VAR_DNSTAP_ENABLE 327
+#define VAR_DNSTAP_SOCKET_PATH 328
+#define VAR_DNSTAP_IP 329
+#define VAR_DNSTAP_TLS 330
+#define VAR_DNSTAP_TLS_SERVER_NAME 331
+#define VAR_DNSTAP_TLS_CERT_BUNDLE 332
+#define VAR_DNSTAP_TLS_CLIENT_KEY_FILE 333
+#define VAR_DNSTAP_TLS_CLIENT_CERT_FILE 334
+#define VAR_DNSTAP_SEND_IDENTITY 335
+#define VAR_DNSTAP_SEND_VERSION 336
+#define VAR_DNSTAP_IDENTITY 337
+#define VAR_DNSTAP_VERSION 338
+#define VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES 339
+#define VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES 340
+#define VAR_REMOTE_CONTROL 341
+#define VAR_CONTROL_ENABLE 342
+#define VAR_CONTROL_INTERFACE 343
+#define VAR_CONTROL_PORT 344
+#define VAR_SERVER_KEY_FILE 345
+#define VAR_SERVER_CERT_FILE 346
+#define VAR_CONTROL_KEY_FILE 347
+#define VAR_CONTROL_CERT_FILE 348
+#define VAR_KEY 349
+#define VAR_ALGORITHM 350
+#define VAR_SECRET 351
+#define VAR_TLS_AUTH 352
+#define VAR_TLS_AUTH_DOMAIN_NAME 353
+#define VAR_TLS_AUTH_CLIENT_CERT 354
+#define VAR_TLS_AUTH_CLIENT_KEY 355
+#define VAR_TLS_AUTH_CLIENT_KEY_PW 356
+#define VAR_PATTERN 357
+#define VAR_NAME 358
+#define VAR_ZONEFILE 359
+#define VAR_NOTIFY 360
+#define VAR_PROVIDE_XFR 361
+#define VAR_ALLOW_QUERY 362
+#define VAR_AXFR 363
+#define VAR_UDP 364
+#define VAR_NOTIFY_RETRY 365
+#define VAR_ALLOW_NOTIFY 366
+#define VAR_REQUEST_XFR 367
+#define VAR_ALLOW_AXFR_FALLBACK 368
+#define VAR_OUTGOING_INTERFACE 369
+#define VAR_ANSWER_COOKIE 370
+#define VAR_COOKIE_SECRET 371
+#define VAR_COOKIE_SECRET_FILE 372
+#define VAR_MAX_REFRESH_TIME 373
+#define VAR_MIN_REFRESH_TIME 374
+#define VAR_MAX_RETRY_TIME 375
+#define VAR_MIN_RETRY_TIME 376
+#define VAR_MIN_EXPIRE_TIME 377
+#define VAR_MULTI_PRIMARY_CHECK 378
+#define VAR_SIZE_LIMIT_XFR 379
+#define VAR_ZONESTATS 380
+#define VAR_INCLUDE_PATTERN 381
+#define VAR_STORE_IXFR 382
+#define VAR_IXFR_SIZE 383
+#define VAR_IXFR_NUMBER 384
+#define VAR_CREATE_IXFR 385
+#define VAR_CATALOG 386
+#define VAR_CATALOG_MEMBER_PATTERN 387
+#define VAR_CATALOG_PRODUCER_ZONE 388
+#define VAR_ZONE 389
+#define VAR_RRL_WHITELIST 390
+#define VAR_SERVERS 391
+#define VAR_BINDTODEVICE 392
+#define VAR_SETFIB 393
+#define VAR_VERIFY 394
+#define VAR_ENABLE 395
+#define VAR_VERIFY_ZONE 396
+#define VAR_VERIFY_ZONES 397
+#define VAR_VERIFIER 398
+#define VAR_VERIFIER_COUNT 399
+#define VAR_VERIFIER_FEED_ZONE 400
+#define VAR_VERIFIER_TIMEOUT 401
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 union YYSTYPE
 {
-#line 47 "configparser.y"
+#line 48 "configparser.y"
 
   char *str;
   long long llng;
@@ -357,8 +365,9 @@ union YYSTYPE
   struct cpu_option *cpu;
   char **strv;
   struct component *comp;
+  int role;
 
-#line 362 "configparser.h"
+#line 371 "configparser.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
@@ -369,6 +378,8 @@ typedef union YYSTYPE YYSTYPE;
 
 extern YYSTYPE c_lval;
 
+
 int c_parse (void);
 
+
 #endif /* !YY_C_CONFIGPARSER_H_INCLUDED  */
