@@ -74,6 +74,7 @@
 #include "remote.h"
 #include "util.h"
 #include "xfrd.h"
+#include "xfrd-catalog-zones.h"
 #include "xfrd-notify.h"
 #include "xfrd-tcp.h"
 #include "nsd.h"
@@ -101,8 +102,10 @@
 #define REMOTE_CONTROL_TCP_TIMEOUT 120
 
 /** repattern to master or slave */
-#define REPAT_SLAVE  1
-#define REPAT_MASTER 2
+#define REPAT_SLAVE                   1
+#define REPAT_MASTER                  2
+#define REPAT_CATALOG_CONSUMER        4
+#define REPAT_CATALOG_CONSUMER_DEINIT 8
 
 /** if you want zero to be inhibited in stats output.
  * it omits zeroes for types that have no acronym and unused-rcodes */
@@ -133,9 +136,6 @@ struct rc_state {
 	struct daemon_remote* rc;
 	/** stats list next item */
 	struct rc_state* stats_next;
-	/** stats list indicator (0 is not part of stats list, 1 is stats,
-	 * 2 is stats_noreset. */
-	int in_stats_list;
 };
 
 /**
@@ -165,8 +165,6 @@ struct daemon_remote {
 	int max_active;
 	/** current commpoints busy; double linked, malloced */
 	struct rc_state* busy_list;
-	/** commpoints waiting for stats to complete (also in busy_list) */
-	struct rc_state* stats_list;
 	/** last time stats was reported */
 	struct timeval stats_time, boot_time;
 #ifdef HAVE_SSL
@@ -223,6 +221,10 @@ remote_accept_callback(int fd, short event, void* arg);
 static void
 remote_control_callback(int fd, short event, void* arg);
 
+#ifdef BIND8_STATS
+/* process the statistics and output them */
+static void process_stats(RES* ssl, xfrd_state_type* xfrd, int peek);
+#endif
 
 /** ---- end of private defines ---- **/
 
@@ -666,7 +668,8 @@ remote_accept_callback(int fd, short event, void* arg)
 		n->ssl = SSL_new(rc->ctx);
 		if(!n->ssl) {
 			log_crypto_err("could not SSL_new");
-			event_del(&n->c);
+			if(n->event_added)
+				event_del(&n->c);
 			free(n);
 			goto close_exit;
 		}
@@ -674,7 +677,8 @@ remote_accept_callback(int fd, short event, void* arg)
 		(void)SSL_set_mode(n->ssl, SSL_MODE_AUTO_RETRY);
 		if(!SSL_set_fd(n->ssl, newfd)) {
 			log_crypto_err("could not SSL_set_fd");
-			event_del(&n->c);
+			if(n->event_added)
+				event_del(&n->c);
 			SSL_free(n->ssl);
 			free(n);
 			goto close_exit;
@@ -686,7 +690,6 @@ remote_accept_callback(int fd, short event, void* arg)
 
 	n->rc = rc;
 	n->stats_next = NULL;
-	n->in_stats_list = 0;
 	n->prev = NULL;
 	n->next = rc->busy_list;
 	if(n->next) n->next->prev = n;
@@ -707,35 +710,10 @@ state_list_remove_elem(struct rc_state** list, struct rc_state* todel)
 	if(todel->next) todel->next->prev = todel->prev;
 }
 
-/** delete from stats list */
-static void
-stats_list_remove_elem(struct rc_state** list, struct rc_state* todel)
-{
-	struct rc_state* prev = NULL;
-	struct rc_state* n = *list;
-	while(n) {
-		/* delete this one? */
-		if(n == todel) {
-			if(prev) prev->next = n->next;
-			else	(*list) = n->next;
-			/* go on and delete further elements */
-			/* prev = prev; */
-			n = n->next;
-			continue;
-		}
-
-		/* go to the next element */
-		prev = n;
-		n = n->next;
-	}
-}
-
 /** decrease active count and remove commpoint from busy list */
 static void
 clean_point(struct daemon_remote* rc, struct rc_state* s)
 {
-	if(s->in_stats_list)
-		stats_list_remove_elem(&rc->stats_list, s);
 	state_list_remove_elem(&rc->busy_list, s);
 	rc->active --;
 	if(s->event_added)
@@ -985,7 +963,7 @@ do_transfer(RES* ssl, xfrd_state_type* xfrd, char* arg)
 			xfrd_handle_notify_and_start_xfr(zone, NULL);
 			send_ok(ssl);
 		} else {
-			(void)ssl_printf(ssl, "error zone not slave\n");
+			(void)ssl_printf(ssl, "error zone not secondary\n");
 		}
 	} else {
 		RBTREE_FOR(zone, xfrd_zone_type*, xfrd->zones) {
@@ -1026,7 +1004,7 @@ do_force_transfer(RES* ssl, xfrd_state_type* xfrd, char* arg)
 			force_transfer_zone(zone);
 			send_ok(ssl);
 		} else {
-			(void)ssl_printf(ssl, "error zone not slave\n");
+			(void)ssl_printf(ssl, "error zone not secondary\n");
 		}
 	} else {
 		RBTREE_FOR(zone, xfrd_zone_type*, xfrd->zones) {
@@ -1064,6 +1042,62 @@ print_zonestatus(RES* ssl, xfrd_state_type* xfrd, struct zone_options* zo)
 		if(!ssl_printf(ssl, "	pattern: %s\n", zo->pattern->pname))
 			return 0;
 	}
+	if(zone_is_catalog_consumer(zo)) {
+		zone_type* zone = namedb_find_zone(xfrd->nsd->db,
+				(const dname_type*)zo->node.key);
+		struct xfrd_catalog_consumer_zone* consumer_zone =
+			(struct xfrd_catalog_consumer_zone*)
+			rbtree_search( xfrd->catalog_consumer_zones
+			             , zo->node.key);
+
+		if(!ssl_printf(ssl, "	catalog: consumer"))
+			return 0;
+		if(zone && zone->soa_rrset && zone->soa_rrset->rrs
+		&& zone->soa_rrset->rrs[0].rdata_count > 2
+		&& rdata_atom_size(zone->soa_rrset->rrs[0].rdatas[2]) ==
+							sizeof(uint32_t)) {
+			if(!ssl_printf(ssl, " (serial: %u, # members: %zu)\n",
+					read_uint32(rdata_atom_data(
+					zone->soa_rrset->rrs[0].rdatas[2])),
+					  consumer_zone
+					? consumer_zone->member_ids.count : 0))
+				return 0;
+
+		} else if(!ssl_printf(ssl, "\n"))
+			return 0;
+		if(invalid_catalog_consumer_zone(zo)) {
+			if(!ssl_printf(ssl, "	catalog-invalid: %s\n",
+					invalid_catalog_consumer_zone(zo)))
+				return 0;
+		}
+	}
+	if(zone_is_catalog_producer(zo)) {
+		struct xfrd_catalog_producer_zone* producer_zone =
+			(struct xfrd_catalog_producer_zone*)
+			rbtree_search( xfrd->catalog_producer_zones
+			             , zo->node.key);
+		if(!ssl_printf(ssl, "	catalog: producer"))
+			return 0;
+		if(producer_zone) {
+			if(!ssl_printf(ssl, " (serial: %u, # members: %zu)\n",
+					(unsigned)producer_zone->serial,
+				       	producer_zone->member_ids.count))
+				return 0;
+		} else if(!ssl_printf(ssl, "\n"))
+			return 0;
+		if (zone_is_slave(zo)) {
+			if(!ssl_printf(ssl, "	catalog-invalid: a catalog "
+					"producer cannot be a secondary zone"))
+				return 0;
+		}
+	}
+	if(zone_is_catalog_member(zo)) {
+		if(!ssl_printf(ssl, "	catalog-member-id: %s\n",
+		   as_catalog_member_zone(zo)->member_id
+		 ? dname_to_string(as_catalog_member_zone(zo)->member_id, NULL)
+		 : "ERROR member-id is missing!"))
+			return 0;
+	}
 	if(nz) {
 		if(nz->is_waiting) {
 			if(!ssl_printf(ssl, "	notify: \"waiting-for-fd\"\n"))
@@ -1084,7 +1118,7 @@ print_zonestatus(RES* ssl, xfrd_state_type* xfrd, struct zone_options* zo)
 		}
 	}
 	if(!xz) {
-		if(!ssl_printf(ssl, "	state: master\n"))
+		if(!ssl_printf(ssl, "	state: primary\n"))
 			return 0;
 		return 1;
 	}
@@ -1229,27 +1263,13 @@ do_status(RES* ssl, xfrd_state_type* xfrd)
 
 /** do the stats command */
 static void
-do_stats(struct daemon_remote* rc, int peek, struct rc_state* rs)
+do_stats(RES* ssl, xfrd_state_type* xfrd, int peek)
 {
 #ifdef BIND8_STATS
-	/* queue up to get stats after a reload is done (to gather statistics
-	 * from the servers) */
-	assert(!rs->in_stats_list);
-	if(peek) rs->in_stats_list = 2;
-	else	rs->in_stats_list = 1;
-	rs->stats_next = rc->stats_list;
-	rc->stats_list = rs;
-	/* block the tcp waiting for the reload */
-	event_del(&rs->c);
-	rs->event_added = 0;
-	/* force a reload */
-	xfrd_set_reload_now(xfrd);
+	process_stats(ssl, xfrd, peek);
 #else
-	RES res;
-	res.ssl = rs->ssl;
-	res.fd = rs->fd;
-	(void)rc; (void)peek;
-	(void)ssl_printf(&res, "error no stats enabled at compile time\n");
+	(void)xfrd; (void)peek;
+	(void)ssl_printf(ssl, "error no stats enabled at compile time\n");
 #endif /* BIND8_STATS */
 }
 
@@ -1308,6 +1328,15 @@ perform_changezone(RES* ssl, xfrd_state_type* xfrd, char* arg)
 			dname = NULL;
 			return 0;
 		}
+		if(zone_is_catalog_consumer_member(zopt)) {
+			(void)ssl_printf(ssl, "Error: Zone is a catalog "
+			  "consumer member zone with id %s\nRepattern in the "
+			  "catalog with a group property.\n", dname_to_string(
+			  as_catalog_member_zone(zopt)->member_id, NULL));
+			region_recycle(xfrd->region, (void*)dname, dname_total_size(dname));
+			dname = NULL;
+			return 0;
+		}
 		/* found the zone, now delete it */
 		/* create deletion task */
 		/* this deletion task is processed before the addition task,
@@ -1322,6 +1351,10 @@ perform_changezone(RES* ssl, xfrd_state_type* xfrd, char* arg)
 			xfrd_del_slave_zone(xfrd, dname);
 		}
 		xfrd_del_notify(xfrd, dname);
+		/* delete it in xfrd's catalog consumers list */
+		if(zone_is_catalog_consumer(zopt)) {
+			xfrd_deinit_catalog_consumer_zone(xfrd, dname);
+		}
 		/* delete from config */
 		zone_list_del(xfrd->nsd->options, zopt);
 	} else {
@@ -1331,7 +1364,8 @@ perform_changezone(RES* ssl, xfrd_state_type* xfrd, char* arg)
 	dname = NULL;
 
 	/* add to zonelist and adds to config in memory */
-	zopt = zone_list_add(xfrd->nsd->options, arg, arg2);
+	zopt = zone_list_add_or_cat(xfrd->nsd->options, arg, arg2,
+			xfrd_add_catalog_producer_member);
 	if(!zopt) {
 		/* also dname parse error here */
 		(void)ssl_printf(ssl, "error could not add zonelist entry\n");
@@ -1343,6 +1377,10 @@ perform_changezone(RES* ssl, xfrd_state_type* xfrd, char* arg)
 		getzonestatid(xfrd->nsd->options, zopt));
 	zonestat_inc_ifneeded(xfrd);
 	xfrd_set_reload_now(xfrd);
+	/* add to xfrd - catalog consumer zones */
+	if (zone_is_catalog_consumer(zopt)) {
+		xfrd_init_catalog_consumer_zone(xfrd, zopt);
+	}
 	/* add to xfrd - notify (for master and slaves) */
 	init_notify_send(xfrd->notify_zones, xfrd->region, zopt);
 	/* add to xfrd - slave */
@@ -1394,7 +1432,8 @@ perform_addzone(RES* ssl, xfrd_state_type* xfrd, char* arg)
 	dname = NULL;
 
 	/* add to zonelist and adds to config in memory */
-	zopt = zone_list_add(xfrd->nsd->options, arg, arg2);
+	zopt = zone_list_add_or_cat(xfrd->nsd->options, arg, arg2,
+			xfrd_add_catalog_producer_member);
 	if(!zopt) {
 		/* also dname parse error here */
 		(void)ssl_printf(ssl, "error could not add zonelist entry\n");
@@ -1406,6 +1445,10 @@ perform_addzone(RES* ssl, xfrd_state_type* xfrd, char* arg)
 		getzonestatid(xfrd->nsd->options, zopt));
 	zonestat_inc_ifneeded(xfrd);
 	xfrd_set_reload_now(xfrd);
+	/* add to xfrd - catalog consumer zones */
+	if (zone_is_catalog_consumer(zopt)) {
+		xfrd_init_catalog_consumer_zone(xfrd, zopt);
+	}
 	/* add to xfrd - notify (for master and slaves) */
 	init_notify_send(xfrd->notify_zones, xfrd->region, zopt);
 	/* add to xfrd - slave */
@@ -1421,6 +1464,8 @@ perform_delzone(RES* ssl, xfrd_state_type* xfrd, char* arg)
 {
 	const dname_type* dname;
 	struct zone_options* zopt;
+	/* dont recycle dname when it becomes part of a xfrd_producer_member */
+	int recycle_dname = 1;
 
 	dname = dname_parse(xfrd->region, arg);
 	if(!dname) {
@@ -1447,7 +1492,17 @@ perform_delzone(RES* ssl, xfrd_state_type* xfrd, char* arg)
 			"nsd.conf yourself and repattern\n");
 		return 0;
 	}
+	if(zone_is_catalog_consumer_member(zopt)
+	&& as_catalog_member_zone(zopt)->member_id) {
+		(void)ssl_printf(ssl, "Error: Zone is a catalog consumer "
+		  "member zone with id %s\nRemove the member id from the "
+		  "catalog to delete this zone.\n", dname_to_string(
+		  as_catalog_member_zone(zopt)->member_id, NULL));
+		region_recycle(xfrd->region, (void*)dname, dname_total_size(dname));
+		dname = NULL;
+		return 0;
 
+	}
 	/* create deletion task */
 	task_new_del_zone(xfrd->nsd->task[xfrd->nsd->mytask],
 		xfrd->last_task, dname);
@@ -1457,10 +1512,18 @@ perform_delzone(RES* ssl, xfrd_state_type* xfrd, char* arg)
 		xfrd_del_slave_zone(xfrd, dname);
 	}
 	xfrd_del_notify(xfrd, dname);
+	/* delete it in xfrd's catalog consumers list */
+	if(zone_is_catalog_consumer(zopt)) {
+		xfrd_deinit_catalog_consumer_zone(xfrd, dname);
+	} else {
+		recycle_dname = !xfrd_del_catalog_producer_member(xfrd, dname);
+	}
 	/* delete from config */
 	zone_list_del(xfrd->nsd->options, zopt);
 
-	region_recycle(xfrd->region, (void*)dname, dname_total_size(dname));
+	if(recycle_dname)
+		region_recycle(xfrd->region,
+				(void*)dname, dname_total_size(dname));
 	return 1;
 }
 
@@ -1621,6 +1684,10 @@ remove_cfgzone(xfrd_state_type* xfrd, const char* pname)
 		xfrd_del_slave_zone(xfrd, dname);
 	}
 	xfrd_del_notify(xfrd, dname);
+	/* delete it in xfrd's catalog consumers list */
+	if(zone_is_catalog_consumer(zopt)) {
+		xfrd_deinit_catalog_consumer_zone(xfrd, dname);
+	}
 
 	/* delete from zoneoptions */
 	zone_options_delete(xfrd->nsd->options, zopt);
@@ -1655,6 +1722,10 @@ add_cfgzone(xfrd_state_type* xfrd, const char* pname)
 		getzonestatid(xfrd->nsd->options, zopt));
 	/* zonestat_inc is done after the entire config file has been done */
 	xfrd_set_reload_now(xfrd);
+	/* add to xfrd - catalog consumer zones */
+	if (zone_is_catalog_consumer(zopt)) {
+		xfrd_init_catalog_consumer_zone(xfrd, zopt);
+	}
 	/* add to xfrd - notify (for master and slaves) */
 	init_notify_send(xfrd->notify_zones, xfrd->region, zopt);
 	/* add to xfrd - slave */
@@ -1803,12 +1874,19 @@ repat_patterns(xfrd_state_type* xfrd, struct nsd_options* newopt)
 			} else if (!p->request_xfr && origp->request_xfr) {
 				newstate = REPAT_MASTER;
 			}
+			if (   p->catalog_role == CATALOG_ROLE_CONSUMER
+			&& origp->catalog_role != CATALOG_ROLE_CONSUMER) {
+				newstate |= REPAT_CATALOG_CONSUMER;
+			} else if (p->catalog_role != CATALOG_ROLE_CONSUMER
+			    && origp->catalog_role == CATALOG_ROLE_CONSUMER) {
+				newstate |= REPAT_CATALOG_CONSUMER_DEINIT;
+			}
 			add_pat(xfrd, p);
 			if (p->implicit && newstate) {
 				const dname_type* dname =
 					parse_implicit_name(xfrd, p->pname);
 				if (dname) {
-					if (newstate == REPAT_SLAVE) {
+					if (newstate & REPAT_SLAVE) {
 						struct zone_options* zopt =
 							zone_options_find(
 							oldopt, dname);
@@ -1816,10 +1894,22 @@ repat_patterns(xfrd_state_type* xfrd, struct nsd_options* newopt)
 							xfrd_init_slave_zone(
 								xfrd, zopt);
 						}
-					} else if (newstate == REPAT_MASTER) {
+					} else if (newstate & REPAT_MASTER) {
 						xfrd_del_slave_zone(xfrd,
 							dname);
 					}
+					if (newstate & REPAT_CATALOG_CONSUMER) {
+						struct zone_options* zopt =
+							zone_options_find(
+							oldopt, dname);
+						if (zopt) {
+							xfrd_init_catalog_consumer_zone(
+								xfrd, zopt);
+						}
+					} else if (newstate & REPAT_CATALOG_CONSUMER_DEINIT) {
+						xfrd_deinit_catalog_consumer_zone(
+								xfrd, dname);
+					}
 					region_recycle(xfrd->region,
 						(void*)dname,
 						dname_total_size(dname));
@@ -1838,15 +1928,23 @@ repat_patterns(xfrd_state_type* xfrd, struct nsd_options* newopt)
 		RBTREE_FOR(zone_opt, struct zone_options*, oldopt->zone_options) {
 			struct pattern_options* oldp = zone_opt->pattern;
 			if (!oldp->implicit) {
-				if (oldp->xfrd_flags == REPAT_SLAVE) {
+				if (oldp->xfrd_flags & REPAT_SLAVE) {
 					/* xfrd needs stable reference so get
 					 * it from the oldopt(modified) tree */
 					xfrd_init_slave_zone(xfrd, zone_opt);
-				} else if (oldp->xfrd_flags == REPAT_MASTER) {
+				} else if (oldp->xfrd_flags & REPAT_MASTER) {
 					xfrd_del_slave_zone(xfrd,
 						(const dname_type*)
 						zone_opt->node.key);
 				}
+				if (oldp->xfrd_flags & REPAT_CATALOG_CONSUMER) {
+					xfrd_init_catalog_consumer_zone(xfrd,
+							zone_opt);
+				} else if (oldp->xfrd_flags & REPAT_CATALOG_CONSUMER_DEINIT) {
+					xfrd_deinit_catalog_consumer_zone(xfrd,
+						(const dname_type*)
+						zone_opt->node.key);
+				}
 				oldp->xfrd_flags = 0;
 			}
 		}
@@ -1924,7 +2022,8 @@ do_repattern(RES* ssl, xfrd_state_type* xfrd)
 
 	(void)ssl_printf(ssl, "reconfig start, read %s\n", cfgfile);
 	opt = nsd_options_create(region);
-	if(!parse_options_file(opt, cfgfile, &print_ssl_cfg_err, &ssl)) {
+	if(!parse_options_file(opt, cfgfile, &print_ssl_cfg_err, &ssl,
+				xfrd->nsd->options)) {
 		/* error already printed */
 		region_destroy(region);
 		return;
@@ -2359,7 +2458,7 @@ cmdcmp(char* p, const char* cmd, size_t len)
 
 /** execute a remote control command */
 static void
-execute_cmd(struct daemon_remote* rc, RES* ssl, char* cmd, struct rc_state* rs)
+execute_cmd(struct daemon_remote* rc, RES* ssl, char* cmd)
 {
 	char* p = skipwhite(cmd);
 	/* compare command */
@@ -2372,9 +2471,9 @@ execute_cmd(struct daemon_remote* rc, RES* ssl, char* cmd, struct rc_state* rs)
 	} else if(cmdcmp(p, "status", 6)) {
 		do_status(ssl, rc->xfrd);
 	} else if(cmdcmp(p, "stats_noreset", 13)) {
-		do_stats(rc, 1, rs);
+		do_stats(ssl, rc->xfrd, 1);
 	} else if(cmdcmp(p, "stats", 5)) {
-		do_stats(rc, 0, rs);
+		do_stats(ssl, rc->xfrd, 0);
 	} else if(cmdcmp(p, "log_reopen", 10)) {
 		do_log_reopen(ssl, rc->xfrd);
 	} else if(cmdcmp(p, "addzone", 7)) {
@@ -2487,7 +2586,7 @@ handle_req(struct daemon_remote* rc, struct rc_state* s, RES* res)
 	VERBOSITY(0, (LOG_INFO, "control cmd: %s", buf));
 
 	/* figure out what to do */
-	execute_cmd(rc, res, buf, s);
+	execute_cmd(rc, res, buf);
 }
 
 #ifdef HAVE_SSL
@@ -2502,7 +2601,8 @@ remote_handshake_later(struct daemon_remote* rc, struct rc_state* s, int fd,
 			return;
 		}
 		s->shake_state = rc_hs_read;
-		event_del(&s->c);
+		if(s->event_added)
+			event_del(&s->c);
 		memset(&s->c, 0, sizeof(s->c));
 		event_set(&s->c, fd, EV_PERSIST|EV_TIMEOUT|EV_READ,
 			remote_control_callback, s);
@@ -2510,6 +2610,7 @@ remote_handshake_later(struct daemon_remote* rc, struct rc_state* s, int fd,
 			log_msg(LOG_ERR, "remote_accept: cannot set event_base");
 		if(event_add(&s->c, &s->tval) != 0)
 			log_msg(LOG_ERR, "remote_accept: cannot add event");
+		s->event_added = 1;
 		return;
 	} else if(r2 == SSL_ERROR_WANT_WRITE) {
 		if(s->shake_state == rc_hs_write) {
@@ -2517,7 +2618,8 @@ remote_handshake_later(struct daemon_remote* rc, struct rc_state* s, int fd,
 			return;
 		}
 		s->shake_state = rc_hs_write;
-		event_del(&s->c);
+		if(s->event_added)
+			event_del(&s->c);
 		memset(&s->c, 0, sizeof(s->c));
 		event_set(&s->c, fd, EV_PERSIST|EV_TIMEOUT|EV_WRITE,
 			remote_control_callback, s);
@@ -2525,6 +2627,7 @@ remote_handshake_later(struct daemon_remote* rc, struct rc_state* s, int fd,
 			log_msg(LOG_ERR, "remote_accept: cannot set event_base");
 		if(event_add(&s->c, &s->tval) != 0)
 			log_msg(LOG_ERR, "remote_accept: cannot add event");
+		s->event_added = 1;
 		return;
 	} else {
 		if(r == 0)
@@ -2590,10 +2693,8 @@ remote_control_callback(int fd, short event, void* arg)
 	res.fd = fd;
 	handle_req(rc, s, &res);
 
-	if(!s->in_stats_list) {
-		VERBOSITY(3, (LOG_INFO, "remote control operation completed"));
-		clean_point(rc, s);
-	}
+	VERBOSITY(3, (LOG_INFO, "remote control operation completed"));
+	clean_point(rc, s);
 }
 
 #ifdef BIND8_STATS
@@ -2746,7 +2847,8 @@ resize_zonestat(xfrd_state_type* xfrd, size_t num)
 }
 
 static void
-zonestat_print(RES* ssl, xfrd_state_type* xfrd, int clear)
+zonestat_print(RES* ssl, xfrd_state_type* xfrd, int clear,
+	struct nsdst** zonestats)
 {
 	struct zonestatname* n;
 	struct nsdst stat0, stat1;
@@ -2761,8 +2863,8 @@ zonestat_print(RES* ssl, xfrd_state_type* xfrd, int clear)
 		 * the newly forked processes get the other block to use,
 		 * these blocks are mmapped and are currently in use to
 		 * add statistics to */
-		memcpy(&stat0, &xfrd->nsd->zonestat[0][n->id], sizeof(stat0));
-		memcpy(&stat1, &xfrd->nsd->zonestat[1][n->id], sizeof(stat1));
+		memcpy(&stat0, &zonestats[0][n->id], sizeof(stat0));
+		memcpy(&stat1, &zonestats[1][n->id], sizeof(stat1));
 		stats_add(&stat0, &stat1);
 		
 		/* save a copy of current (cumulative) stats in stat1 */
@@ -2798,7 +2900,8 @@ zonestat_print(RES* ssl, xfrd_state_type* xfrd, int clear)
 #endif /* USE_ZONE_STATS */
 
 static void
-print_stats(RES* ssl, xfrd_state_type* xfrd, struct timeval* now, int clear)
+print_stats(RES* ssl, xfrd_state_type* xfrd, struct timeval* now, int clear,
+	struct nsdst* st, struct nsdst** zonestats)
 {
 	size_t i;
 	stc_type total = 0;
@@ -2825,9 +2928,9 @@ print_stats(RES* ssl, xfrd_state_type* xfrd, struct timeval* now, int clear)
 		return;
 
 	/* mem info, database on disksize */
-	if(!print_longnum(ssl, "size.db.disk=", xfrd->nsd->st.db_disk))
+	if(!print_longnum(ssl, "size.db.disk=", st->db_disk))
 		return;
-	if(!print_longnum(ssl, "size.db.mem=", xfrd->nsd->st.db_mem))
+	if(!print_longnum(ssl, "size.db.mem=", st->db_mem))
 		return;
 	if(!print_longnum(ssl, "size.xfrd.mem=", region_get_mem(xfrd->region)))
 		return;
@@ -2837,64 +2940,153 @@ print_stats(RES* ssl, xfrd_state_type* xfrd, struct timeval* now, int clear)
 	if(!print_longnum(ssl, "size.config.mem=", region_get_mem(
 		xfrd->nsd->options->region)))
 		return;
-	print_stat_block(ssl, "", "", &xfrd->nsd->st);
+	print_stat_block(ssl, "", "", st);
 
 	/* zone statistics */
+	if(!ssl_printf(ssl, "zone.primary=%lu\n",
+		(unsigned long)(xfrd->notify_zones->count - xfrd->zones->count)))
+		return;
+	if(!ssl_printf(ssl, "zone.secondary=%lu\n", (unsigned long)xfrd->zones->count))
+		return;
 	if(!ssl_printf(ssl, "zone.master=%lu\n",
 		(unsigned long)(xfrd->notify_zones->count - xfrd->zones->count)))
 		return;
 	if(!ssl_printf(ssl, "zone.slave=%lu\n", (unsigned long)xfrd->zones->count))
 		return;
 #ifdef USE_ZONE_STATS
-	zonestat_print(ssl, xfrd, clear); /* per-zone statistics */
+	zonestat_print(ssl, xfrd, clear, zonestats); /* per-zone statistics */
 #else
-	(void)clear;
+	(void)clear; (void)zonestats;
 #endif
 }
 
+/* allocate stats temp arrays, for taking a coherent snapshot of the
+ * statistics values at that time. */
 static void
-clear_stats(xfrd_state_type* xfrd)
+process_stats_alloc(xfrd_state_type* xfrd, struct nsdst** stats,
+	struct nsdst** zonestats)
+{
+	*stats = xmallocarray(xfrd->nsd->child_count*2, sizeof(struct nsdst));
+#ifdef USE_ZONE_STATS
+	zonestats[0] = xmallocarray(xfrd->zonestat_safe, sizeof(struct nsdst));
+	zonestats[1] = xmallocarray(xfrd->zonestat_safe, sizeof(struct nsdst));
+#else
+	(void)zonestats;
+#endif
+}
+
+/* grab a copy of the statistics, at this particular time. */
+static void
+process_stats_grab(xfrd_state_type* xfrd, struct timeval* stattime,
+	struct nsdst* stats, struct nsdst** zonestats)
+{
+	if(gettimeofday(stattime, NULL) == -1)
+		log_msg(LOG_ERR, "gettimeofday: %s", strerror(errno));
+	memcpy(stats, xfrd->nsd->stat_map,
+		xfrd->nsd->child_count*2*sizeof(struct nsdst));
+#ifdef USE_ZONE_STATS
+	memcpy(zonestats[0], xfrd->nsd->zonestat[0],
+		xfrd->zonestat_safe*sizeof(struct nsdst));
+	memcpy(zonestats[1], xfrd->nsd->zonestat[1],
+		xfrd->zonestat_safe*sizeof(struct nsdst));
+#else
+	(void)zonestats;
+#endif
+}
+
+/* add the old and new processes stat values into the first part of the
+ * array of stats */
+static void
+process_stats_add_old_new(xfrd_state_type* xfrd, struct nsdst* stats)
 {
 	size_t i;
-	uint64_t dbd = xfrd->nsd->st.db_disk;
-	uint64_t dbm = xfrd->nsd->st.db_mem;
+	uint64_t dbd = stats[0].db_disk;
+	uint64_t dbm = stats[0].db_mem;
+	/* The old and new server processes have separate stat blocks,
+	 * and these are added up together. This results in the statistics
+	 * values per server-child. The reload task briefly forks both
+	 * old and new server processes. */
 	for(i=0; i<xfrd->nsd->child_count; i++) {
-		xfrd->nsd->children[i].query_count = 0;
+		stats_add(&stats[i], &stats[xfrd->nsd->child_count+i]);
 	}
-	memset(&xfrd->nsd->st, 0, sizeof(struct nsdst));
-	/* zonestats are cleared by storing the cumulative value that
-	 * was last printed in the zonestat_clear array, and subtracting
-	 * that before the next stats printout */
-	xfrd->nsd->st.db_disk = dbd;
-	xfrd->nsd->st.db_mem = dbm;
+	stats[0].db_disk = dbd;
+	stats[0].db_mem = dbm;
 }
 
-void
-daemon_remote_process_stats(struct daemon_remote* rc)
+/* manage clearing of stats, a cumulative count of cleared statistics */
+static void
+process_stats_manage_clear(xfrd_state_type* xfrd, struct nsdst* stats,
+	int peek)
 {
-	RES res;
-	struct rc_state* s;
-	struct timeval now;
-	if(!rc) return;
-	if(gettimeofday(&now, NULL) == -1)
-		log_msg(LOG_ERR, "gettimeofday: %s", strerror(errno));
-	/* pop one and give it stats */
-	while((s = rc->stats_list)) {
-		assert(s->in_stats_list);
-#ifdef HAVE_SSL
-		res.ssl = s->ssl;
-#endif
-		res.fd = s->fd;
-		print_stats(&res, rc->xfrd, &now, (s->in_stats_list == 1));
-		if(s->in_stats_list == 1) {
-			clear_stats(rc->xfrd);
-			rc->stats_time = now;
+	struct nsdst st;
+	size_t i;
+	if(peek) {
+		/* Subtract the earlier resetted values from the numbers,
+		 * but do not reset the values that are retrieved now. */
+		if(!xfrd->stat_clear)
+			return; /* nothing to subtract */
+		for(i=0; i<xfrd->nsd->child_count; i++) {
+			/* subtract cumulative count that has been reset */
+			stats_subtract(&stats[i], &xfrd->stat_clear[i]);
 		}
-		VERBOSITY(3, (LOG_INFO, "remote control stats printed"));
-		rc->stats_list = s->next;
-		s->in_stats_list = 0;
-		clean_point(rc, s);
+		return;
 	}
+	if(!xfrd->stat_clear)
+		xfrd->stat_clear = region_alloc_zero(xfrd->region,
+			sizeof(struct nsdst)*xfrd->nsd->child_count);
+	for(i=0; i<xfrd->nsd->child_count; i++) {
+		/* store cumulative count copy */
+		memcpy(&st, &stats[i], sizeof(st));
+		/* subtract cumulative count that has been reset */
+		stats_subtract(&stats[i], &xfrd->stat_clear[i]);
+		/* store cumulative count in the cleared value array */
+		memcpy(&xfrd->stat_clear[i], &st, sizeof(st));
+	}
+}
+
+/* add up the statistics to get the total over the server children. */
+static void
+process_stats_add_total(xfrd_state_type* xfrd, struct nsdst* total,
+	struct nsdst* stats)
+{
+	size_t i;
+	/* copy over the first one, with also the nonadded values. */
+	memcpy(total, &stats[0], sizeof(*total));
+	xfrd->nsd->children[0].query_count = stats[0].qudp + stats[0].qudp6
+		+ stats[0].ctcp + stats[0].ctcp6 + stats[0].ctls
+		+ stats[0].ctls6;
+	for(i=1; i<xfrd->nsd->child_count; i++) {
+		stats_add(total, &stats[i]);
+		xfrd->nsd->children[i].query_count = stats[i].qudp
+			+ stats[i].qudp6 + stats[i].ctcp + stats[i].ctcp6
+			+ stats[i].ctls + stats[i].ctls6;
+	}
+}
+
+/* process the statistics and output them */
+static void
+process_stats(RES* ssl, xfrd_state_type* xfrd, int peek)
+{
+	struct timeval stattime;
+	struct nsdst* stats, *zonestats[2], total;
+
+	process_stats_alloc(xfrd, &stats, zonestats);
+	process_stats_grab(xfrd, &stattime, stats, zonestats);
+	process_stats_add_old_new(xfrd, stats);
+	process_stats_manage_clear(xfrd, stats, peek);
+	process_stats_add_total(xfrd, &total, stats);
+	print_stats(ssl, xfrd, &stattime, !peek, &total, zonestats);
+	if(!peek) {
+		xfrd->nsd->rc->stats_time = stattime;
+	}
+
+	free(stats);
+#ifdef USE_ZONE_STATS
+	free(zonestats[0]);
+	free(zonestats[1]);
+#endif
+
+	VERBOSITY(3, (LOG_INFO, "remote control stats printed"));
 }
 #endif /* BIND8_STATS */
 
